SSP = {}
SSP.cache = {}

SSP.language = (GetCVar("language.2") or "en")

function SSP.GetTotalSpentPoints()
	if not SSP.cache.total then
		SSP.SetTotalSpentPoints()
	end
	return SSP.cache.total
end

function SSP.SetTotalSpentPoints()
	SSP.cache.skillTypes = {}
	SSP.cache.total = 0
	local num = GetNumSkillTypes()
	
	for i=1,num do
		SSP.cache.total = SSP.cache.total + SSP.SetTypeSpentPoints( i )
	end
	
	return SSP.cache.total
end

function SSP.GetTypeSpentPoints( skillType )
	if not SSP.cache.skillTypes then
		SSP.GetTotalSpentPoints()
	end
	
	if not SSP.cache.skillTypes[ skillType ] then
		SSP.SetTypeSpentPoints( skillType )
	end
	return SSP.cache.skillTypes[ skillType ].total
end

function SSP.SetTypeSpentPoints( skillTypeId )
	SSP.cache.skillTypes[ skillTypeId ] = {}
	local skillType = SSP.cache.skillTypes[ skillTypeId ]
	skillType.lines = {}
	skillType.total = 0
	local num = GetNumSkillLines( skillTypeId )
	
	for i=1,num do
		skillType.total = skillType.total + SSP.SetLineSpentPoints( skillTypeId, i )
	end
	
	return skillType.total
end

function SSP.GetLineSpentPoints( skillType, skillLine )
	if not SSP.cache.skillTypes then
		SSP.GetTotalSpentPoints()
	end
	
	if not SSP.cache.skillTypes[ skillType ] then
		SSP.SetTypeSpentPoints( skillType )
	end
	
	if not SSP.cache.skillTypes[ skillType ].lines[skillLine] then
		SSP.SetLineSpentPoints( skillType, skillLine )
	end
	
	return SSP.cache.skillTypes[ skillType ].lines[skillLine].total
end

function SSP.SetLineSpentPoints( skillType, skillLine )
	SSP.cache.skillTypes[ skillType ].lines[skillLine] = {}
	local line = SSP.cache.skillTypes[ skillType ].lines[skillLine]
	line.skills = {}
	line.total = 0
	line.possible = 0
	local num = GetNumSkillAbilities(skillType, skillLine )
	local spent
	for i=1,num do
		spent = SSP.SetSkillSpentPoints( skillType, skillLine, i )
		line.total = line.total + spent[1]
		line.possible = line.possible + spent[2]
	end
	
	return line.total
end

function SSP.GetLinePossiblePoints( skillType, skillLine )
	-- this is only called to refresh the values
	local total = SSP.GetLineSpentPoints( skillType, skillLine )
	
	return SSP.cache.skillTypes[ skillType ].lines[skillLine].possible
end

function SSP.GetSkillSpentPoints( skillType, skillLine, skillIndex )
	if not SSP.cache.skillTypes then
		SSP.GetTotalSpentPoints()
	end
	
	if not SSP.cache.skillTypes[ skillType ] then
		SSP.SetTypeSpentPoints( skillType )
	end
	
	if not SSP.cache.skillTypes[ skillType ].lines[skillLine] then
		SSP.SetLineSpentPoints( skillType, skillLine )
	end
	
	if not SSP.cache.skillTypes[ skillType ].lines[skillLine].skills[skillIndex] then
		SSP.SetSkillSpentPoints( skillType, skillLine, skillIndex )
	end
	
	return SSP.cache.skillTypes[ skillType ].lines[skillLine].skills[skillIndex]
end

function SSP.SetSkillSpentPoints( skillType, skillLine, skillIndex )
	local skills = SSP.cache.skillTypes[ skillType ].lines[skillLine].skills
	local _, _, _, _, _, purchased, progressionIndex = GetSkillAbilityInfo( skillType, skillLine, skillIndex )
	local spent
	local possible
	local reduction
	
	if not purchased then
		spent = 0
	end
	
	reduction = SSP.ReduceAbility( skillType, skillLine, skillIndex )
	if progressionIndex then
		local _, morph = GetAbilityProgressionInfo( progressionIndex )
		if not spent then
			if morph > 0 then
				spent = 2 - reduction
			else
				spent = 1 - reduction
			end
		end
		possible = 2 - reduction
	else
		local currentLevel, maxLevel = GetSkillAbilityUpgradeInfo( skillType, skillLine, skillIndex )
		if currentLevel then
			if not spent then spent = currentLevel - reduction end
			possible = maxLevel - reduction
		else
			if not spent then spent = 1 - reduction end
			possible = 1 - reduction
		end
	end
	
	skills[skillIndex] = {spent, possible}
	return skills[skillIndex]
end

function SSP.ReduceAbility( skillType, skillLine, skillIndex )
	if skillType == SKILL_TYPE_WORLD and skillLine == 1 and skillIndex == 2 then
		--soul magic
		return 1
	end
	if skillType == SKILL_TYPE_TRADESKILL then
		if skillIndex == 1 then
			return 1 --first trade skill is always free
		end
		
		if SSP.language ==  "de" then
			if (skillLine == 6 or skillLine == 5) and skillIndex == 2 then
				return 1 --echantment, provision has another free skill
			end
		elseif SSP.language ==  "fr" then
			if (skillLine == 3 or skillLine == 5) and skillIndex == 2 then
				return 1 --echantment, provision has another free skill
			end
		else -- language == "en", but better don't check and use it as substitute for other languages.
			if (skillLine == 4 or skillLine == 5) and skillIndex == 2 then
				return 1 --echantment, provision has another free skill
			end
		end
	end
	if skillType == SKILL_TYPE_RACIAL and skillLine <= 10 and skillIndex == 1 then
		-- exp bonus is free
		return 1
	end
	return 0
end

function SSP.OnSkillPointsChanged( _, oldPoints, newPoints, oldSkyshards, newSkyshards )
	if oldPoints ~= newPoints then
		SSP.SetTotalSpentPoints()
	end
end

function SSP.ToggleColor( ... )
	SSP.settings.color = not SSP.settings.color
	if SSP.settings.color then
		d("Enabled colors for SpentSkillPoints.")
	else
		d("Disabled colors for SpentSkillPoints.")
	end
end

function SSP.OnLoad(eventCode, addOnName)
	if addOnName ~= "SpentSkillPoints" then
		return
	end
	SSP.cache = {}--
	if not SSP.cache.total then
		SSP.SetTotalSpentPoints()
	end
	
	SLASH_COMMANDS["/ssp_color"] = SSP.ToggleColor
	
	SSP.settings = ZO_SavedVars:New("SSP_SavedVars", 2, "settings", {color = false})
	
	EVENT_MANAGER:RegisterForEvent("SpentSkillPoints", EVENT_SKILL_POINTS_CHANGED, SSP.OnSkillPointsChanged)
	
end

EVENT_MANAGER:RegisterForEvent("SpentSkillPoints", EVENT_ADD_ON_LOADED, SSP.OnLoad)